<?php
/**
 * Mono functions and definitions
 *
 * @package Mono
 */

/**
 * The theme version.
 */
define( 'MONO_VERSION', '1.2.1' );

/**
 * Set the content width based on the theme's design and stylesheet.
 */
if ( ! isset( $content_width ) ) {
	$content_width = 770; /* pixels */
}

/**
 * The suffix to use for scripts.
 */
if ( ( defined( 'SCRIPT_DEBUG' ) && true === SCRIPT_DEBUG ) ) {
	define( 'MONO_SUFFIX', '' );
} else {
	define( 'MONO_SUFFIX', '.min' );
}

/**
 * Set the content width in pixels, based on the theme's design and stylesheet.
 *
 * Priority 0 to make it available to lower priority callbacks.
 *
 * @global int $content_width
 */
function mono_content_width() {
	$GLOBALS['content_width'] = apply_filters( 'mono_content_width', 770 );
}
add_action( 'after_setup_theme', 'mono_content_width', 0 );

if ( ! function_exists( 'mono_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 *
	 * Note that this function is hooked into the after_setup_theme hook, which
	 * runs before the init hook. The init hook is too late for some features, such
	 * as indicating support for post thumbnails.
	 *
	 * @since 1.0.0.
	 *
	 * @return void
	 */
	function mono_setup() {
		/*
		 * Make theme available for translation.
		 * Translations can be filed in the /languages/ directory.
		 * If you're building a theme based on mono, use a find and replace
		 * to change 'mono' to the name of your theme in all the template files
		 */
		load_theme_textdomain( 'mono', get_template_directory() . '/languages' );

		// Add default posts and comments RSS feed links to head.
		add_theme_support( 'automatic-feed-links' );

		/*
		 * Let WordPress manage the document title.
		 * By adding theme support, we declare that this theme does not use a
		 * hard-coded <title> tag in the document head, and expect WordPress to
		 * provide it for us.
		 */
		add_theme_support( 'title-tag' );

		/*
		 * Enable support for Post Thumbnails on posts and pages.
		 *
		 * @link http://codex.wordpress.org/Function_Reference/add_theme_support#Post_Thumbnails
		 */
		add_theme_support( 'post-thumbnails' );

		// Set up the WordPress core custom background feature.
		add_theme_support(
			'custom-background',
			apply_filters(
				'mono_custom_background_args',
				array(
					'default-color' => 'ffffff',
					'default-image' => '',
				)
			)
		);

		add_theme_support(
			'custom-logo',
			array(
				'header-text' => array( 'site-title', 'site-description' ),
			)
		);

		// This theme uses wp_nav_menu() in two locations.
		register_nav_menus(
			array(
				'primary' => esc_html__( 'Header Menu', 'mono' ),
				'footer'  => esc_html__( 'Footer', 'mono' ),
			)
		);

		/*
		 * Switch default core markup for search form, comment form, and comments
		 * to output valid HTML5.
		 */
		add_theme_support(
			'html5',
			array(
				'search-form',
				'comment-form',
				'comment-list',
				'gallery',
				'caption',
			)
		);

		/**
		 * Add Gutenberg Support.
		 */
		add_theme_support( 'align-wide' );
		add_theme_support( 'wp-block-styles' );
		add_theme_support( 'responsive-embeds' );
		add_theme_support( 'editor-styles' );
		add_editor_style( '/assets/css/editor-style.css' );
	}
endif; // mono_setup.

add_action( 'after_setup_theme', 'mono_setup' );

/**
 * Load global variables used on the front end.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function mono_frontend_globals() {
	global $mono_settings;
	$mono_settings = new MONO_Settings();
}

add_action( 'template_redirect', 'mono_frontend_globals' );

/**
 * Register widget area.
 *
 * @link http://codex.wordpress.org/Function_Reference/register_sidebar
 *
 * @since 1.0.0.
 *
 * @return void
 */
function mono_widgets_init() {
	register_sidebar(
		array(
			'name'          => esc_html__( 'Footer', 'mono' ),
			'id'            => 'sidebar-1',
			'description'   => esc_html__( 'This widget area will appear in footer of your site.', 'mono' ),
			'before_widget' => '<aside id="%1$s" class="widget grid__col grid__col--1-of-3 %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h4 class="widget-title">',
			'after_title'   => '</h4>',
		)
	);
}

add_action( 'widgets_init', 'mono_widgets_init' );

/**
 * Enqueue Gutenberg Styles & Scripts.
 */
function mono_block_editor_assets() {
	global $mono_settings;

	// Initialize an Instance of Theme Settings Object.
	$mono_settings = new MONO_Settings();

	// Getting & Setting Google fonts.
	$google_request = mono_get_google_font_uri();

	if ( '' !== $google_request ) {
		// Enqueue the fonts.
		wp_enqueue_style(
			'mono-google-fonts',
			$google_request,
			array(),
			MONO_VERSION
		);
	}

	$font_header = $mono_settings->get_value( 'font-headers' );
	$font_body   = $mono_settings->get_value( 'font-body' );

	$accent_color   = $mono_settings->get_value( 'color-accent' );
	$style_bg_color = $mono_settings->get_value( 'background_color' );

	wp_add_inline_style(
		'mono-google-fonts',
		"
		.editor-styles-wrapper {
			--accent-color: {$accent_color};
			--style-background-color: {$style_bg_color};
			--font-body: '{$font_body}';
			--font-header: '{$font_header}';
		}"
	);
}
add_action( 'enqueue_block_editor_assets', 'mono_block_editor_assets' );

/**
 * Enqueue scripts and styles.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function mono_scripts() {
	$style_dependencies = array();

	// Google fonts.
	$google_request = mono_get_google_font_uri();
	if ( '' !== $google_request ) {
		// Enqueue the fonts.
		wp_enqueue_style(
			'mono-google-fonts',
			$google_request,
			$style_dependencies,
			MONO_VERSION
		);
		$style_dependencies[] = 'mono-google-fonts';
	}

	// Main stylesheet.
	wp_enqueue_style(
		'mono-style',
		get_stylesheet_uri(),
		$style_dependencies,
		MONO_VERSION
	);
	$style_dependencies[] = 'mono-style';

	if ( ! wp_style_is( 'genericons', 'enqueued' ) ) {
		wp_enqueue_style( 'genericons', get_template_directory_uri() . '/assets/css/libs/genericons/genericons.css', array(), '3.4.1' );
	}

	$script_dependencies = array( 'jquery' );

	$script_dependencies[] = 'masonry';

	wp_enqueue_script(
		'modernizr-custom',
		get_template_directory_uri() . '/assets/js/modernizr.custom.js',
		array( 'jquery' ),
		'2.7.1',
		true
	);
	$script_dependencies[] = 'modernizr-custom';

	// Front end script in footer.
	wp_enqueue_script(
		'mono-frontend-footer',
		get_template_directory_uri() . '/assets/js/frontend-footer.js',
		$script_dependencies,
		MONO_VERSION,
		true
	);

	// Comment reply script.
	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		wp_enqueue_script( 'comment-reply' );
	}
}

add_action( 'wp_enqueue_scripts', 'mono_scripts' );

/**
 * Register the required plugins for this theme.
 *
 * @since 1.0.0
 *
 * @return void
 */
function mono_required_plugins() {
	$plugins = array(
		array(
			'name'     => __( 'Jetpack', 'mono' ),
			'slug'     => 'jetpack',
			'required' => true,
		),
		array(
			'name'     => __( 'StagTools', 'mono' ),
			'slug'     => 'stagtools',
			'required' => true,
		),
		array(
			'name'     => __( 'Stag Custom Sidebars', 'mono' ),
			'slug'     => 'stag-custom-sidebars',
			'required' => true,
		),
		array(
			'name'             => __( 'Mono Assistant', 'mono' ),
			'slug'             => 'mono-assistant',
			'source'           => get_parent_theme_file_path() . '/assistant/mono-assistant.zip',
			'required'         => true,
			'force_activation' => true,
			'version'          => '1.0',
			'external_url'     => 'https://github.com/codestag/mono-assistant/',
		),
		array(
			'name'         => __( 'Envato Market', 'mono' ),
			'slug'         => 'wp-envato-market',
			'source'       => esc_url( 'https://envato.github.io/wp-envato-market/dist/envato-market.zip' ),
			'required'     => false,
			'version'      => '2.0.1',
			'external_url' => esc_url( 'https://github.com/envato/wp-envato-market' ),
		),
	);

	tgmpa( $plugins );
}
add_action( 'tgmpa_register', 'mono_required_plugins' );

if ( class_exists( 'Subtitles' ) && method_exists( 'Subtitles', 'subtitle_styling' ) ) {
	remove_action( 'wp_head', array( Subtitles::getInstance(), 'subtitle_styling' ) );
}

if ( ! function_exists( 'mono_assistant_notice' ) ) {
	/**
	 * Alerts if assistant plugin is not activated.
	 *
	 * @since 1.2.0
	 * @return void
	 */
	function mono_assistant_notice() {
		if ( ! class_exists( 'Mono_Assistant' ) && current_user_can( 'activate_plugins' ) ) {
			$page_link = self_admin_url( 'themes.php?page=tgmpa-install-plugins' );
			$message   = '<p>' . sprintf( '%1$s<br><strong>%2$s<a href="%3$s">this page</a>.', __( 'As per ThemeForest guidelines, some functionalities are now excluded from theme and moved to an assistant plugin.', 'mono' ), __( 'To use Mono with full features, please activate Mono Assistant from ', 'mono' ), esc_url( $page_link ) ) . '</strong></p>';

			echo '<div class="error">' . $message . '</div>'; // phpcs:ignore
		}
	}
}
add_action( 'admin_notices', 'mono_assistant_notice' );

/* Load theme files */
require get_template_directory() . '/inc/settings.php';
require get_template_directory() . '/inc/customizer/load.php';
require get_template_directory() . '/inc/css.php';
require get_template_directory() . '/inc/style.php';
require get_template_directory() . '/inc/class-tgm-plugin-activation.php';

// Miscellaneous.
require get_template_directory() . '/inc/jetpack.php';
require get_template_directory() . '/inc/extras.php';
require get_template_directory() . '/inc/template-tags.php';

/**
* Include theme updater file
*/
require_once get_template_directory() . '/inc/updater/theme-updater.php';
