<?php
/**
 * Mono Theme Customizer
 *
 * @package Mono
 */

/**
 * Add postMessage support for site title and description for the Theme Customizer.
 *
 * @since 1.0.0.
 *
 * @param WP_Customize_Manager $wp_customize Theme Customizer object.
 *
 * @return void
 */
function mono_customize_register( $wp_customize ) {
	// Tagline transport.
	$wp_customize->get_setting( 'blogdescription' )->transport = 'postMessage';
}

add_action( 'customize_register', 'mono_customize_register' );

/**
 * Enqueue scripts to run in the Customizer preview pane.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function mono_customize_preview_js() {
	wp_enqueue_script(
		'mono-customizer-preview',
		get_template_directory_uri() . '/assets/js/customizer-preview.js',
		array( 'customize-preview' ),
		MONO_VERSION,
		true
	);

	wp_localize_script(
		'mono-customizer-preview',
		'monoPreview',
		array(
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
		)
	);
}

add_action( 'customize_preview_init', 'mono_customize_preview_js' );

function mono_customize_controls_enqueue_scripts() {
	// Styles.
	wp_enqueue_style(
		'select2-css',
		get_template_directory_uri() . '/assets/css/libs/select2/select2' . MONO_SUFFIX . '.css',
		array(),
		'4.0.13'
	);

	// Attach Select2 Customizer Z-index patch.
	wp_add_inline_style(
		'select2-css',
		'.select2-container {
					z-index: 600000;
				}
			'
	);

	// Scripts.
	wp_enqueue_script(
		'select2-js',
		get_template_directory_uri() . '/assets/js/libs/select2/select2' . MONO_SUFFIX . '.js',
		array( 'jquery', 'customize-controls' ),
		'4.0.13',
		true
	);

	wp_enqueue_script(
		'customizer-sections',
		get_template_directory_uri() . '/assets/js/customizer-sections.js',
		array( 'customize-controls', 'select2-js' ),
		MONO_VERSION,
		true
	);

	$localize = array(
		'fontOptions'               => array_merge( mono_get_font_property_option_keys(), array( 'google-font-subset' ) ),
	);

	// Localize the script.
	wp_localize_script(
		'customizer-sections',
		'select2Options',
		$localize
	);
}

add_action( 'customize_controls_enqueue_scripts', 'mono_customize_controls_enqueue_scripts' );


/**
 * Add sections and controls to the customizer.
 *
 * @since  1.0.0.
 *
 * @param  WP_Customize_Manager $wp_customize    The Customizer instance.
 * @return void
 */
function mono_customize_add_sections( $wp_customize ) {
	// Compile the section definitions.
	$sections = mono_customize_get_definitions( $wp_customize );

	// Initial priority.
	$priority = new MONO_Prioritizer( 1000 );

	// Register each section and add its options.
	foreach ( $sections as $section => $data ) {
		// Store the options.
		if ( isset( $data['options'] ) ) {
			$options = $data['options'];
			unset( $data['options'] );
		}

		// Determine the priority.
		if ( ! isset( $data['priority'] ) ) {
			$data['priority'] = $priority->add();
		}

		// Register section, if it doesn't already exist.
		if ( ! $wp_customize->get_section( $section ) ) {
			$wp_customize->add_section( $section, $data );
		}

		// Add options to the section.
		if ( isset( $options ) ) {
			mono_customize_add_section_options( $section, $options );
			unset( $options );
		}
	}
}

add_action( 'customize_register', 'mono_customize_add_sections' );

/**
 * Register settings and controls for a section.
 *
 * @since  1.0.0.
 *
 * @param  string $section             Section ID
 * @param  array  $args                Array of setting and control definitions
 * @param  int    $initial_priority    The initial priority to use for controls
 * @return int                            The last priority value assigned
 */
function mono_customize_add_section_options( $section, $args, $initial_priority = 10 ) {
	global $wp_customize;

	$settings = new MONO_Settings();
	$priority = new MONO_Prioritizer( $initial_priority, 10 );

	foreach ( $args as $setting_id => $option ) {
		// Add setting.
		if ( isset( $option['setting'] ) ) {
			$defaults = array(
				'type'                 => 'theme_mod',
				'capability'           => 'edit_theme_options',
				'theme_supports'       => '',
				'default'              => $settings->get_default( $setting_id ),
				'transport'            => 'refresh',
				'sanitize_callback'    => $settings->get_sanitize_callback( $setting_id ),
				'sanitize_js_callback' => '',
			);
			$setting  = wp_parse_args( $option['setting'], $defaults );

			// Add the setting arguments inline so Theme Check can verify the presence of sanitize_callback.
			$wp_customize->add_setting(
				$setting_id,
				array(
					'type'                 => $setting['type'],
					'capability'           => $setting['capability'],
					'theme_supports'       => $setting['theme_supports'],
					'default'              => $setting['default'],
					'transport'            => $setting['transport'],
					'sanitize_callback'    => $setting['sanitize_callback'],
					'sanitize_js_callback' => $setting['sanitize_js_callback'],
				)
			);
		}

		// Add control.
		if ( isset( $option['control'] ) ) {
			$control_id = $setting_id;

			$defaults = array(
				'settings' => $setting_id,
				'section'  => $section,
				'priority' => $priority->add(),
			);

			if ( ! isset( $option['setting'] ) ) {
				unset( $defaults['settings'] );
			}

			$control = wp_parse_args( $option['control'], $defaults );

			// Check for a specialized control class.
			if ( isset( $control['control_class'] ) ) {
				$class = $control['control_class'];

				/**
				 * Filter the path for loading a Customizer control file.
				 *
				 * @since 1.0.0.
				 *
				 * @param string    $control_path    The path to the control file.
				 * @param string    $control_id      The ID of the current control.
				 * @param array     $control         The array of parameters for the current control.
				 */
				$control_path = apply_filters( 'mono_customizer_control_path', get_template_directory() . '/inc/customizer/controls', $control_id, $control );

				$control_file = trailingslashit( $control_path ) . $class . '.php';
				if ( file_exists( $control_file ) ) {
					require_once $control_file;
				}

				if ( class_exists( $class ) ) {
					unset( $control['control_type'] );

					// Dynamically generate a new class instance in a way that's compatible with PHP 5.2.
					$reflection     = new ReflectionClass( $class );
					$class_instance = $reflection->newInstanceArgs( array( $wp_customize, $control_id, $control ) );

					$wp_customize->add_control( $class_instance );
				}
			} else {
				$wp_customize->add_control( $control_id, $control );
			}
		}
	}

	return $priority->get();
}
