<?php
/**
 * Custom functions that act independently of the theme templates
 *
 * Eventually, some of the functionality here could be replaced by core features
 *
 * @package Mono
 */

/**
 * Adds custom classes to the array of body classes.
 *
 * @since 1.0.0.
 *
 * @param array $classes Classes for the body element.
 *
 * @return array
 */
function mono_body_classes( $classes ) {
	// Adds a class if the body is being rendered in the Customizer preview pane.
	if ( is_customize_preview() ) {
		$classes[] = 'wp-customizer-preview';
	}

	// Adds a class of group-blog to blogs with more than 1 published author.
	if ( is_multi_author() ) {
		$classes[] = 'group-blog';
	}

	global $mono_settings;

	if ( true === $mono_settings->get_value( 'use-background-cover' ) ) {
		$classes[] = 'use-background-cover';
	}

	if ( true === $mono_settings->get_value( 'center-post-cover' ) ) {
		$classes[] = 'centered-post-cover';
	}

	return $classes;
}

add_filter( 'body_class', 'mono_body_classes' );

/**
 * Add a wrapper div to the output of oembeds and the [embed] shortcode.
 *
 * Also enqueues FitVids, since the embed might be a video.
 *
 * @since 1.0.0.
 *
 * @param  string $html    The generated HTML of the embed handler.
 * @param  string $url     The embed URL.
 * @param  array  $attr    The attributes of the embed shortcode.
 *
 * @return string             The wrapped HTML.
 */
function mono_embed_container( $html, $url, $attr ) {
	// Bail if this is the admin or an RSS feed.
	if ( is_admin() || is_feed() ) {
		return $html;
	}

	// Add FitVids as a dependency for the Frontend footer script.
	global $wp_scripts;
	if ( is_object( $wp_scripts ) && 'WP_Dependencies' == get_class( $wp_scripts ) ) {
		$script = $wp_scripts->query( 'mono-frontend-footer', 'registered' );
		if ( $script && ! in_array( 'fitvids', $script->deps ) ) {
			$script->deps[] = 'fitvids';
		}
	}

	if ( isset( $attr['width'] ) ) {
		// Get classes.
		$default_class = 'mono-embed-wrapper';
		$align_class   = 'aligncenter';
		if ( isset( $attr['mono_align'] ) ) {
			$align = trim( $attr['mono_align'] );
			if ( in_array( $align, array( 'left', 'right' ) ) ) {
				$align_class = 'align' . $align;
			}
		}
		$class = "$default_class $align_class";
		// Get style.
		$style = 'max-width: ' . absint( $attr['width'] ) . 'px;';
		// Build wrapper.
		$wrapper = "<div class=\"$class\" style=\"$style\">%s</div>";
		$html    = sprintf( $wrapper, $html );
	}

	return $html;
}

add_filter( 'embed_handler_html', 'mono_embed_container', 10, 3 );
add_filter( 'embed_oembed_html', 'mono_embed_container', 10, 3 );
