<?php
/**
 * @package Mono
 */

/**
 * Return an object containing an array of CSS rules.
 *
 * This function behaves differently depending on which parameters are given.
 * - With no parameters, it returns the rules for all the CSS-related settings using their current stored values.
 * - With an array of setting IDs, it returns the rules for those specific settings, using their current stored values.
 * - With both an array of setting IDs and a corresponding array of values, it will return the rules for those specific settings, using the given values (sanitized).
 *
 * @since 1.0.0.
 *
 * @param  array $setting_ids       An optional array of setting IDs.
 * @param  array $preview_values    An optional array of values for the settings.
 *
 * @return mixed|MONO_CSS|void
 */
function mono_get_css_rules( $setting_ids = array(), $preview_values = array() ) {
	$settings      = new MONO_Settings();
	$css           = new MONO_CSS();
	$setting_pairs = array();

	// Default array of settings.
	if ( empty( $setting_ids ) ) {
		$setting_ids = array(
			'background_color',
			'color-accent',
			'color-footer-background',
			'font-body',
			'font-headers',
			'bool-hide-credit',
			'custom-settings',
		);
	}

	// Use the saved setting values if no preview values are given.
	if ( empty( $preview_values ) ) {
		foreach ( (array) $setting_ids as $id ) {
			$setting_pairs[ $id ] = $settings->get_value( $id );
		}
	} else {
		$setting_pairs = array_combine( (array) $setting_ids, (array) $preview_values );
		array_walk( $setting_pairs, array( $settings, 'sanitize_value' ) );
	}

	// Iterate through each setting.
	foreach ( $setting_pairs as $setting_id => $sanitized_value ) {
		/**
		 * Filter to override the CSS rules for a particular setting.
		 *
		 * To override the CSS rules for a setting, create a function that accepts all three
		 * of the parameters of this filter. Use the MONO_CSS methods in the $css parameter
		 * to add custom rules, then return $css instead of $override.
		 *
		 * Example:
		 * function child_css_override_titlecolor( $override, $setting_id, $css ) {
		 *     if ( 'color-site-title' === $setting_id ) {
		 *         $css->add( array(
		 *             'selectors'    => array(
		 *                 '.site-title', '.site-title a',
		 *             ),
		 *             'declarations' => array(
		 *                 'background-color' => $settings->get_value( $setting_id ),
		 *             ),
		 *         ) );
		 *         return $css;
		 *     }
		 *
		 *     return $override;
		 * }
		 * add_filter( 'mono_css_rule_override', 'child_css_override_titlecolor', 10, 4 );
		 *
		 * To add rules for a setting not in the defaults list, or a new setting, use the 'custom-settings'
		 * value for the setting ID, like this:
		 *
		 *     if ( 'custom-settings' === $setting_id ) {
		 *         // Rules go here.
		 *     }
		 *
		 * @since 1.0.0.
		 *
		 * @param bool|MONO_CSS    $override           Value determines whether setting CSS rules get overridden.
		 * @param string               $setting_id         The ID of the setting to override.
		 * @param mixed                $sanitized_value    The sanitized value currently assigned to the setting.
		 * @param MONO_CSS         $css                The instance of MONO_CSS.
		 */
		$override = apply_filters( 'mono_css_rule_override', false, $setting_id, $sanitized_value, $css );
		if ( false !== $override && 'MONO_CSS' === get_class( $override ) ) {
			$css = $override;
			continue;
		}

		switch ( $setting_id ) {
			case 'font-body':
				$css->add(
					array(
						'selectors'    => array(
							'body',
						),
						'declarations' => array(
							'font-family' => "'$sanitized_value'",
						),
					)
				);
				break;
			case 'font-headers':
				$css->add(
					array(
						'selectors'    => array(
							'.site-branding',
							'h1,h2,h3,h4,h5,h6',
							'.wp-block-quote p',
							'.wp-block-pullquote p',
						),
						'declarations' => array(
							'font-family' => "'$sanitized_value'",
						),
					)
				);
				break;
			case 'color-accent':
				$css->add(
					array(
						'selectors'    => array(
							'a',
						),
						'declarations' => array(
							'color' => $sanitized_value,
						),
					)
				);
				$css->add(
					array(
						'selectors'    => array(
							'button',
							'.button',
							'.twitter-follow-button',
							'.instagram-follow-link',
							'input[type="reset"]',
							'input[type="submit"]',
							'input[type="button"]',
						),
						'declarations' => array(
							'background-color' => $sanitized_value,
						),
					)
				);
				break;
			case 'color-footer-background':
				$css->add(
					array(
						'selectors'    => array(
							'.site-footer',
						),
						'declarations' => array(
							'background-color' => $sanitized_value,
						),
					)
				);
				break;
			case 'bool-hide-credit':
				if ( true === $sanitized_value ) {
					$css->add(
						array(
							'selectors'    => array(
								'.site-byline',
							),
							'declarations' => array(
								'display' => 'none',
							),
						)
					);
				} else {
					$css->add(
						array(
							'selectors'    => array(
								'.site-byline',
							),
							'declarations' => array(
								'display' => 'block',
							),
						)
					);
				}
				break;
		}
	}

	return $css;
}

/**
 * Output the theme CSS rules as an inline style element.
 *
 * Used in the Customizer preview pane.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function mono_inline_css_rules() {
	// Get the rules.
	$css = mono_get_css_rules();

	// Build the rules.
	$output = $css->build();

	// Output the rules.
	if ( ! empty( $output ) ) {
		echo "\n<style type=\"text/css\" id=\"mono-inline-css\">\n";
		echo $output; // phpcs:ignore
		echo "\n</style>\n";
	}
}

add_action( 'wp_head', 'mono_inline_css_rules', 99 );

/**
 * Add the theme CSS rules to the content editor.
 *
 * @since 1.0.0.
 *
 * @param  string $stylesheets    The comma-separated string of stylesheet URLs.
 *
 * @return string                    The modified string of stylesheet URLs.
 */
function mono_editor_css_rules( $stylesheets ) {
	$stylesheets .= ',' . add_query_arg( 'action', 'mono-css-rules', admin_url( 'admin-ajax.php' ) );
	return $stylesheets;
}

add_filter( 'mce_css', 'mono_editor_css_rules', 99 );

/**
 * Generates the theme CSS as an Ajax response.
 *
 * @since  1.0.0.
 *
 * @return void
 */
function mono_ajax_css_rules() {
	// Make sure this is an Ajax request.
	if ( ! defined( 'DOING_AJAX' ) || true !== DOING_AJAX ) {
		return;
	}

	/**
	 * Filter whether the dynamic stylesheet will send headers telling the browser
	 * to cache the request. Set to false to turn off these headers.
	 *
	 * @since 1.0.0.
	 *
	 * @param bool    $cache_headers
	 */
	if ( true === apply_filters( 'mono_stylesheet_cache_headers', true ) ) {
		/**
		 * Set headers for caching.
		 *
		 * @link http://stackoverflow.com/a/15000868
		 * @link http://www.mobify.com/blog/beginners-guide-to-http-cache-headers
		 */
		$expires = HOUR_IN_SECONDS;
		header( 'Pragma: public' );
		header( 'Cache-Control: private, max-age=' . $expires );
		header( 'Expires: ' . gmdate( 'D, d M Y H:i:s', time() + $expires ) . ' GMT' );
	}

	// Set header for content type.
	header( 'Content-type: text/css' );

	// Get the rules.
	$css = mono_get_css_rules();

	// Echo the rules.
	echo $css->build(); // phpcs:ignore

	// End the Ajax response.
	die();
}

add_action( 'wp_ajax_mono-css-rules', 'mono_ajax_css_rules' );
add_action( 'wp_ajax_nopriv_mono-css-rules', 'mono_ajax_css_rules' );

/**
 * Returns the theme CSS as a JSON object.
 *
 * Used for asynchronous previewing in the Customizer.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function mono_ajax_css_json() {
	// Make sure this is an Ajax request.
	if ( ! defined( 'DOING_AJAX' ) || true !== DOING_AJAX ) {
		return;
	}

	$settings = new MONO_Settings();

	// Get post parameters.
	$setting_ids = array();
	if ( isset( $_POST['sid'] ) ) {
		$setting_ids = array_map( 'sanitize_title_with_dashes', (array) $_POST['sid'] );
	}
	$values = array();
	if ( isset( $_POST['val'] ) ) {
		foreach ( (array) $_POST['val'] as $index => $raw_value ) {
			$values[ $index ] = $settings->sanitize_value( $raw_value, $setting_ids[ $index ] );
		}
	}

	// Generate the rules.
	$css = mono_get_css_rules( $setting_ids, $values );

	// Send response.
	if ( isset( $css->data ) && is_array( $css->data ) && ! empty( $css->data ) ) {
		wp_send_json_success( $css->data );
	} else {
		wp_send_json_error( $css );
	}
}

add_action( 'wp_ajax_mono-css-json', 'mono_ajax_css_json' );

/**
 * Saves a version number (Unix timestamp) when Customizer settings are saved.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function mono_customize_save_version() {
	set_theme_mod( 'version', time() );
}

add_action( 'customize_save_after', 'mono_customize_save_version' );
