<?php
/**
 * Custom template tags for this theme.
 *
 * Eventually, some of the functionality here could be replaced by core features.
 *
 * @package Mono
 */

if ( ! function_exists( 'mono_posted_on' ) ) :
	/**
	 * Prints HTML with meta information for the current post-date/time and author.
	 * Pluggable.
	 *
	 * @since 1.0.0.
	 *
	 * @return void
	 */
	function mono_posted_on() {
		$time_string = '<time class="entry-date published updated" datetime="%1$s">%2$s</time>';
		if ( get_the_time( 'U' ) !== get_the_modified_time( 'U' ) ) {
			$time_string = '<time class="entry-date published" datetime="%1$s">%2$s</time><time class="updated" datetime="%3$s">%4$s</time>';
		}

		$time_string = sprintf(
			$time_string,
			esc_attr( get_the_date( 'c' ) ),
			esc_html( get_the_date() ),
			esc_attr( get_the_modified_date( 'c' ) ),
			esc_html( get_the_modified_date() )
		);

		$posted_on = sprintf(
			esc_html_x( '%s', 'post date', 'mono' ),
			'<a href="' . esc_url( get_permalink() ) . '" rel="bookmark">' . $time_string . '</a>'
		);

		$byline = sprintf(
			_x( '%s', 'post author', 'mono' ),
			'<span class="author vcard"><a class="url fn n" href="' . esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ) . '">' . esc_html( get_the_author() ) . '</a></span>'
		);

		echo '<span class="posted-on">' . $posted_on . '</span><span class="byline"> ' . $byline . '</span>';

		if ( 'post' == get_post_type() ) {
			/* translators: used between list items, there is a space after the comma */
			$categories_list = get_the_category_list( esc_html__( ', ', 'mono' ) );
			if ( $categories_list && mono_categorized_blog() ) {
				printf( '<span class="cat-links">' . esc_attr__( 'In %1$s', 'mono' ) . '</span>', $categories_list );
			}
		}
	}
endif;

if ( ! function_exists( 'mono_post_nav' ) ) :
	/**
	 * Display navigation to next/previous post when applicable.
	 */
	function mono_post_nav() {
		// Don't print empty markup if there's nowhere to navigate.
		$previous = ( is_attachment() ) ? get_post( get_post()->post_parent ) : get_adjacent_post( false, '', true );
		$next     = get_adjacent_post( false, '', false );

		$prev_subtitle = ( ! empty( get_post_meta( $previous->ID, '_subtitle', true ) ) ) ? get_post_meta( $previous->ID, '_subtitle', true ) : '';
		$next_subtitle = ( ! empty( get_post_meta( $next->ID, '_subtitle', true ) ) ) ? get_post_meta( $next->ID, '_subtitle', true ) : '';

		if ( ! $next && ! $previous ) {
			return;
		}
		?>
	<nav class="navigation post-navigation" role="navigation">
		<div class="container">
			<h1 class="screen-reader-text"><?php esc_html_e( 'Post navigation', 'mono' ); ?></h1>
			<div class="nav-links grid">
				<?php
					previous_post_link(
						'<div class="nav-previous nav-link grid__col grid__col--1-of-2">%link</div>',
						_x( '<span class="meta-nav">Previous Post</span> <h2 class="post-title">%title</h2>', 'Previous post link', 'mono' ) . "<span class='entry-subtitle'>{$prev_subtitle}</span>"
					);

					next_post_link(
						'<div class="nav-next nav-link grid__col grid__col--1-of-2">%link</div>',
						_x( '<span class="meta-nav">Next Post</span> <h2 class="post-title">%title</h2>', 'Next post link', 'mono' ) . "<span class='entry-subtitle'>{$next_subtitle}</span>"
					);
				?>
			</div><!-- .nav-links -->
		</div>
	</nav><!-- .navigation -->
		<?php
	}
endif;

if ( ! function_exists( 'mono_entry_footer' ) ) :
	/**
	 * Prints HTML with meta information for the categories, tags and comments.
	 * Pluggable.
	 *
	 * @since 1.0.0.
	 *
	 * @return void
	 */
	function mono_entry_footer() {
		// Hide category and tag text for pages.
		if ( 'post' == get_post_type() ) {
			/* translators: used between list items, there is a space after the comma */
			$categories_list = get_the_category_list( esc_html__( ', ', 'mono' ) );
			if ( $categories_list && mono_categorized_blog() ) {
				printf( '<span class="cat-links">' . esc_attr__( 'Posted in %1$s', 'mono' ) . '</span>', $categories_list );
			}

			/* translators: used between list items, there is a space after the comma */
			$tags_list = get_the_tag_list( '', esc_html__( ', ', 'mono' ) );
			if ( $tags_list ) {
				printf( '<span class="tags-links">' . esc_attr__( 'Tagged %1$s', 'mono' ) . '</span>', $tags_list );
			}
		}

		if ( ! is_single() && ! post_password_required() && ( comments_open() || get_comments_number() ) ) {
			echo '<span class="comments-link">';
			comments_popup_link( esc_html__( 'Leave a comment', 'mono' ), esc_html__( '1 Comment', 'mono' ), esc_html__( '% Comments', 'mono' ) );
			echo '</span>';
		}
	}
endif;

/**
 * Returns true if a blog has more than 1 category.
 *
 * @since 1.0.0.
 *
 * @return bool
 */
function mono_categorized_blog() {
	if ( false === ( $all_the_cool_cats = get_transient( 'mono_categories' ) ) ) {
		// Create an array of all the categories that are attached to posts.
		$all_the_cool_cats = get_categories(
			array(
				'fields'     => 'ids',
				'hide_empty' => 1,

				// We only need to know if there is more than one category.
				'number'     => 2,
			)
		);

		// Count the number of categories that are attached to the posts.
		$all_the_cool_cats = count( $all_the_cool_cats );

		set_transient( 'mono_categories', $all_the_cool_cats );
	}

	if ( $all_the_cool_cats > 1 ) {
		// This blog has more than 1 category so mono_categorized_blog should return true.
		return true;
	} else {
		// This blog has only 1 category so mono_categorized_blog should return false.
		return false;
	}
}

/**
 * Flush out the transients used in mono_categorized_blog.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function mono_category_transient_flusher() {
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
		return;
	}
	// Like, beat it. Dig?
	delete_transient( 'mono_categories' );
}
add_action( 'edit_category', 'mono_category_transient_flusher' );
add_action( 'save_post', 'mono_category_transient_flusher' );

if ( ! function_exists( 'mono_excerpt_more' ) && ! is_admin() ) :
	/**
	 * Replaces "[...]" (appended to automatically generated excerpts) with ... and
	 * a 'Continue reading' link.
	 *
	 * Create your own mono_excerpt_more() function to override in a child theme.
	 *
	 * @return string 'Continue reading' link prepended with an ellipsis.
	 */
	function mono_excerpt_more() {
		$allowed_tags = mono_allowed_html();
		$link         = sprintf(
			'<a href="%1$s" class="more-link">%2$s</a>',
			esc_url( get_permalink( get_the_ID() ) ),
			/* translators: %s: Name of current post */
			wp_kses( sprintf( __( 'Read More &rarr; <span class="screen-reader-text"> "%s"</span>', 'mono' ), get_the_title( get_the_ID() ) ), $allowed_tags )
		);
		return ' ' . $link;
	}
	add_filter( 'excerpt_more', 'mono_excerpt_more' );
endif;

if ( ! function_exists( 'mono_excerpt_length' ) ) :
	/**
	 * Custom excerpt length.
	 *
	 * @return int Number of words returned as excerpt text.
	 */
	function mono_excerpt_length( $length ) {
		return 20;
	}
endif;
add_filter( 'excerpt_length', 'mono_excerpt_length', 999 );

if ( ! function_exists( 'wp_body_open' ) ) {
	/**
	 * Backwards compatibility for wp_body_open.
	 *
	 * @return void.
	 */
	function wp_body_open() {
		do_action( 'wp_body_open' );
	}
}
